@@ -8,7 +8,7 @@ package Inline::Java ;
 use strict ;
 require 5.6.0 ;
 
-$Inline::Java::VERSION = '0.43' ;
+$Inline::Java::VERSION = '0.44' ;
 
 
 # DEBUG is set via the DEBUG config
@@ -127,6 +127,9 @@ sub validate {
 
 	$o->set_option('EXTRA_JAVA_ARGS',		'',		's', 1, \%opts) ;
 	$o->set_option('EXTRA_JAVAC_ARGS',		'',		's', 1, \%opts) ;
+	$o->set_option('DEBUGGER',				0,		'b', 1, \%opts) ;
+
+	$o->set_option('PRIVATE',				'',		'b', 1, \%opts) ;
 
 	my @left_overs = keys(%opts) ;
 	if (scalar(@left_overs)){
@@ -153,11 +156,27 @@ sub validate {
 	if (($o->get_java_config('JNI'))&&($o->get_java_config('SHARED_JVM'))){
 		croak("You can't use the 'SHARED_JVM' option in 'JNI' mode") ;
 	}
+	if (($o->get_java_config('JNI'))&&($o->get_java_config('DEBUGGER'))){
+		croak("You can't invoke the Java debugger ('DEBUGGER' option) in 'JNI' mode") ;
+	}
 
 	if ($o->get_java_config('JNI')){
 		require Inline::Java::JNI ;
 	}
 
+	if ($o->get_java_config('DEBUGGER')){
+		# Here we want to tweak a few settings to help debugging...
+		Inline::Java::debug(1, "Debugger mode activated") ;
+		# Add the -g compile option
+		$o->set_java_config('EXTRA_JAVAC_ARGS', $o->get_java_config('EXTRA_JAVAC_ARGS') . " -g ") ;
+		# Add the -sourcepath runtime option
+		$o->set_java_config('EXTRA_JAVA_ARGS', $o->get_java_config('EXTRA_JAVA_ARGS') .
+			" -sourcepath " . $o->get_api('build_dir') .
+			portable("ENV_VAR_PATH_SEP_CP") .
+			get_source_dir()
+		) ;
+	}	
+
 	my $study = $o->get_java_config('STUDY') ;
 	if ((defined($study))&&(ref($study) ne 'ARRAY')){
 		croak "Configuration option 'STUDY' must be an array of Java class names" ;
@@ -350,7 +369,7 @@ sub build {
 
 	# Go back and clean up
 	chdir $cwd ;
-	if ($o->get_api('cleanup')){
+	if (($o->get_api('cleanup'))&&(! $o->get_java_config('DEBUGGER'))){
 		$o->rmpath('', $build_dir) ;
 	}
 
@@ -435,29 +454,49 @@ sub load {
 	$o->set_java_config('id', scalar(@INLINES) - 1) ;
 	Inline::Java::debug(3, "Inline::Java object id is " . $o->get_java_config('id')) ;
 
-	my $classes = [] ;
+	$o->study_module() ;
 	if ((defined($o->get_java_config('STUDY')))&&(scalar($o->get_java_config('STUDY')))){
-		$classes = $o->get_java_config('STUDY') ;
+		$o->_study($o->get_java_config('STUDY')) ;
 	}
-	$o->_study($classes, 1) ;
 
 	$o->set_java_config('loaded', 1) ;
 	Inline::Java::debug(1, "load done.") ;
 }
 
 
-# This function 'studies' the specified classes and binds them to 
-# Perl
-sub _study {
+# This function 'studies' the classes generated by the inlined code.
+sub study_module {
 	my $o = shift ;
-	my $classes = shift ;
-	my $study_module = shift ;
 
-	my $install_dir = File::Spec->catdir($o->get_api('install_lib'), 
+	my $install_dir = File::Spec->catdir($o->get_api('install_lib'),
 		'auto', $o->get_api('modpname')) ;
+	my $cache = $o->get_api('modfname') . '.' . $o->get_api('suffix') ;
 
-	if ($study_module){
-		# We need to add the classes that are in the directory or under...
+	my $lines = [] ;
+	if (! $o->get_java_config('built')){
+		# Since we didn't build the module, this means that
+		# it was up to date. We can therefore use the data
+		# from the cache.
+		Inline::Java::debug(1, "using jdat cache") ;
+		my $p = File::Spec->catfile($install_dir, $cache) ;
+		my $size = (-s $p) || 0 ;
+		if ($size > 0){
+			if (open(Inline::Java::CACHE, "<$p")){
+				while (<Inline::Java::CACHE>){
+					push @{$lines}, $_ ;
+				}
+				close(Inline::Java::CACHE) ;
+			}
+			else{
+				croak "Can't open $p for reading: $!" ;
+			}
+		}
+	}
+	else{
+		# First thing to do is get the list of classes that comprise the module.
+
+		# We need the classes that are in the directory or under...
+		my @classes = () ;
 		my $cwd = Cwd::cwd() ;
 		if ($o->get_config('UNTAINT')){
 			($cwd) = $cwd =~ /(.*)/ ;
@@ -469,10 +508,37 @@ sub _study {
 		my @fl = Inline::Java::Portable::find_classes_in_dir('.') ;
 		chdir $cwd ;
 		foreach my $f (@fl){
-			push @{$classes}, $f->{class} ;
+			push @classes, $f->{class} ;
+		}
+
+		# Now we ask Java the info about those classes...
+		$lines = $o->report(@classes) ;
+
+		# and we update the cache with these results.
+		Inline::Java::debug(1, "updating jdat cache") ;
+		my $p = File::Spec->catfile($install_dir, $cache) ;
+		if (open(Inline::Java::CACHE, ">$p")){
+			foreach my $l (@{$lines}){
+				print Inline::Java::CACHE "$l\n" ;
+			}
+			close(Inline::Java::CACHE) ;
+		}
+		else{
+			croak "Can't open $p file for writing" ;
 		}
 	}
 
+	# Now we read up the symbols and bind them to Perl.
+	$o->bind_jdat($o->load_jdat($lines)) ;
+}
+
+
+# This function 'studies' the specified classes and binds them to 
+# Perl.
+sub _study {
+	my $o = shift ;
+	my $classes = shift ;
+
 	my @new_classes = () ;
 	foreach my $class (@{$classes}){
 		$class = Inline::Java::Class::ValidateClass($class) ;
@@ -480,78 +546,37 @@ sub _study {
 			push @new_classes, $class ;
 		}
 	}
-
 	if (! scalar(@new_classes)){
 		return ;
 	}
 	
-	# Then we ask it to give us the public symbols from the classes
-	# that we got.
-	my @lines = $o->report(\@new_classes, $study_module) ;
-
+	my $lines = $o->report(@new_classes) ;
 	# Now we read up the symbols and bind them to Perl.
-	$o->bind_jdat($o->load_jdat(@lines)) ;
+	$o->bind_jdat($o->load_jdat($lines)) ;
 }
 
 
-# This function asks the JVM what are the public symbols for the specified
-# classes
 sub report {
 	my $o = shift ;
-	my $classes = shift ;
-	my $study_module = shift || 0 ;
+	my @classes = @_ ;
 
-	my $install_dir = File::Spec->catdir($o->get_api('install_lib'), 
-		'auto', $o->get_api('modpname')) ;
-	my $cache = $o->get_api('modfname') . '.' . $o->get_api('suffix') ;
-
-	if (($study_module)&&(! $o->get_java_config('built'))){
-		# Since we didn't build the module, this means that 
-		# it was up to date. We can therefore use the data 
-		# from the cache
-		Inline::Java::debug(1, "using jdat cache") ;
-		my $p = File::Spec->catfile($install_dir, $cache) ;
-		my $size = (-s $p) || 0 ;
-		if ($size > 0){
-			if (open(Inline::Java::CACHE, "<$p")){
-				my $resp = join("", <Inline::Java::CACHE>) ;
-				close(Inline::Java::CACHE) ;
-				return split("\n", $resp) ;
-			}
-			else{
-				croak "Can't open $p for reading: $!" ;
-			}
-		}
+	my @lines = () ;
+	if (scalar(@classes)){
+		my $pc = new Inline::Java::Protocol(undef, $o) ;
+		my $resp = $pc->Report(join(" ", @classes)) ;
+		@lines = split("\n", $resp) ;
 	}
 
-	# Ok, there are some classes in there that we don't know about.
-	# Ask for the info on the classes
-	my $pc = new Inline::Java::Protocol(undef, $o) ;
-	my $resp = $pc->Report(join(" ", @{$classes})) ;
-
-	if (($study_module)&&($o->get_java_config('built'))){
-		# Update the cache.
-		Inline::Java::debug(1, "updating jdat cache") ;
-		my $p = File::Spec->catfile($install_dir, $cache) ;
-		if (open(Inline::Java::CACHE, ">$p")){
-			print Inline::Java::CACHE $resp ;
-			close(Inline::Java::CACHE) ;
-		}
-		else{
-			croak "Can't open $p file for writing" ;
-		}
-	}
-
-	return split("\n", $resp) ;
+	return \@lines ;
 }
 
 
 # Load the jdat code information file.
 sub load_jdat {
 	my $o = shift ;
-	my @lines = @_ ;
+	my $lines = shift ;
 
-	Inline::Java::debug(5, join("\n", @lines)) ;
+	Inline::Java::debug_obj($lines) ;
 
 	# We need an array here since the same object can have many 
 	# study sessions.
@@ -566,7 +591,7 @@ sub load_jdat {
 
 	my $idx = 0 ;
 	my $current_class = undef ;
-	foreach my $line (@lines){
+	foreach my $line (@{$lines}){
 		chomp($line) ;
 		if ($line =~ /^class ($re)$/){
 			# We found a class definition
@@ -722,7 +747,7 @@ CODE
 			$code .= $o->bind_method($idx, $class, $method) ;
 		}
 
-		Inline::Java::debug(5, $code) ;
+		Inline::Java::debug_obj(\$code) ;
 
 		# open (Inline::Java::CODE, ">>code") and print CODE $code and close(CODE) ;
 
@@ -1018,7 +1043,7 @@ sub study_classes {
 	}
 
 	if (defined($o)){
-		$o->_study($classes, 0) ;
+		$o->_study($classes) ;
 	}
 	else {
 		my $msg = "Can't place studied classes under package '$package' since Inline::Java was not used there. Valid packages are:\n" ;
